@@ -30,6 +30,8 @@ from pandas.core.groupby.generic import DataFrameGroupBy, SeriesGroupBy
 from pandas.core.groupby.groupby import GroupBy
 from pandas.core.reshape.merge import _MergeOperation
 
+from pandas_tutor.__version__ import __version__
+
 # Literal only added in Python 3.8
 from typing_extensions import Literal, TypeGuard
 
@@ -59,6 +61,10 @@ JSONScalar = Union[int, float, str, bool, None]
 SERIES = "pandas.Series"
 
 
+def in_dev() -> bool:
+    return "dev" in __version__
+
+
 def first(iterable: Iterable[T]) -> T:
     """the first element of an iterable."""
     return next(iter(iterable))
@@ -219,7 +225,7 @@ class CodeRange:
 
 @overload
 def positions_to_labels(
-    positions: Union[int, Label],
+    positions: Label,
     df: HasIndex,
     slicer: Slicer = "iloc",
     axis: Axis = "index",
@@ -290,6 +296,18 @@ def match_cols(
     )
 
 
+def get_pd_from_babypandas(val: Any) -> Any:
+    """
+    gets original pd value out of a babypandas object. if it's not a
+    babypandas object, returns the value itself
+    """
+    return (
+        val._pd
+        if hasattr(val, "_pd") and (is_pd(val._pd) or is_groupby(val._pd))
+        else val
+    )
+
+
 def is_series(obj: Any) -> TypeGuard[pd.Series]:
     return isinstance(obj, pd.Series)
 
@@ -302,6 +320,16 @@ def is_pd(obj: Any) -> TypeGuard[Union[pd.DataFrame, pd.Series]]:
     return is_series(obj) or is_dataframe(obj)
 
 
+def is_scalar(obj: Any) -> TypeGuard[Any]:
+    # np.generic is base class for ALL numpy scalars
+    return isinstance(obj, np.generic) or isinstance(
+        # this list isn't comprehensive (e.g. bytes, bytearray) but it covers
+        # most use cases for pandas
+        obj,
+        (int, float, str, list, tuple, range, dict, set, frozenset),
+    )
+
+
 def is_groupby(obj: Any) -> TypeGuard[Union[DataFrameGroupBy, SeriesGroupBy]]:
     return isinstance(obj, (DataFrameGroupBy, SeriesGroupBy))
 
@@ -491,10 +519,11 @@ def json_scalar(obj: Any) -> JSONScalar:
     """
     if pd.isnull(obj):
         return None
-    if isinstance(obj, np.integer):
-        return int(obj)
-    if isinstance(obj, np.floating):
-        return float(obj)
+    # we need to convert numpy types to python types since our json library
+    # doesn't handle it natively. if we forget about a type, we run into a
+    # circular reference error during dumps()
+    if isinstance(obj, (np.integer, np.bool_, np.floating)):
+        return obj.item()
     if isinstance(obj, np.ndarray):
         return obj.tolist()
     if isinstance(obj, pd.Timestamp) or isinstance(obj, pd.Timedelta):
@@ -526,15 +555,16 @@ def df_data(df: pd.DataFrame) -> List[List[JSONScalar]]:
 
 
 def mem_used(obj: Any) -> float:
-    if isinstance(obj, pd.DataFrame):
-        return obj.memory_usage(deep=True).sum()
-    elif isinstance(obj, pd.Series):
-        return obj.memory_usage(deep=True)
-    else:
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        if isinstance(obj, pd.DataFrame):
+            return obj.memory_usage(deep=True).sum()
+        elif isinstance(obj, pd.Series):
+            return obj.memory_usage(deep=True)
+        else:
             from pympler.asizeof import asizeof
-        return asizeof(obj)
+
+            return asizeof(obj)
 
 
 KB = 2**10
